@@ -1,5 +1,5 @@
 #
-# $Id: RequestParameterPrimary.pl,v 16.2 2001/01/05 21:43:40 wpm Exp $
+# $Id: RequestParameterPrimary.pl,v 17.1 2001/04/06 18:45:07 biersma Exp $
 #
 # (c) 1999-2001 Morgan Stanley Dean Witter and Co.
 # See ..../src/LICENSE for terms of distribution.
@@ -40,6 +40,15 @@ package MQSeries::Command::MQSC;
    InquireChannel		=> "ChannelName",
    InquireChannelStatus		=> "ChannelName",
 
+   #
+   # StorageClass commands
+   #
+   ChangeStorageClass		=> "StorageClassName",
+   #CopyStorageClass		=> "StorageClassName",
+   CreateStorageClass		=> "StorageClassName",
+   DeleteStorageClass		=> "StorageClassName",
+   InquireStorageClass		=> "StorageClassName",
+
   );
 
 1;
