@@ -2,10 +2,10 @@
 # descriptions.pl - Include file for MQSeries::ErrorLog::Parser
 #                   that describes all known error types.
 #
-# (c) 2000-2001 Morgan Stanley Dean Witter and Co.
+# (c) 2000,2001 Morgan Stanley Dean Witter and Co.
 # See ..../src/LICENSE for terms of distribution.
 # 
-# $Id: descriptions.pl,v 16.1 2001/01/05 21:45:51 wpm Exp $
+# $Id: descriptions.pl,v 17.1 2001/05/21 20:16:30 biersma Exp $
 #
 
 #
@@ -23,11 +23,12 @@ use vars qw($error_table);
 #
 my $channel_patt = '[\w\.\%\/]+';
 my $code_patt = '-?\d+';
-my $exit_patt = '/[\w\.\/\-\(\)]+';
+my $exit_patt = '[\w\.\/\-\(\)]+';
 my $hex_patt = '[A-F\d]+';
 my $hostname_patt = '[\w\-]+';
 my $ip_patt = '\d+\.\d+\.\d+\.\d+';
 my $host_patt = "$hostname_patt \\($ip_patt\\) \\(\\d+\\)|$hostname_patt \\($ip_patt\\)|$ip_patt \\(\\d+\\)|$ip_patt";
+my $logfile_patt = 'S\d+\.LOG';
 my $pid_patt = '\d+';
 my $qmgr_patt = '[\w\.!%\/]+';
 my $qname_patt = '[\w\.\%\/]+';
@@ -78,8 +79,8 @@ $error_table =
    'AMQ5817' => [ "An invalid stream queue \\(($qname_patt)\\s*\\) has been detected by the MQSeries message broker\\.",
                   "Stream" ],
 
-   # FIXME: So far, encountered this guy without an actual broker name...
-   'AMQ5818' => [ "Unable to open MQSeries message broker stream queue \\(\\) for reason 0,0" ],
+   # NOTE: Have encountered this guy without an actual broker name...
+   'AMQ5818' => [ "Unable to open MQSeries message broker stream queue \\((.*?)\s*\\) for reason (\\d+,\\d+)\." ],
 
    'AMQ5819' => [ "MQSeries message broker stream \\(($qname_patt)\\s*\\) has ended abnormally for reason ($reason_patt)\\.",
                   "Stream", "Reason" ],
@@ -93,13 +94,23 @@ $error_table =
    'AMQ5822' => [ "MQSeries message broker failed to register as a child of broker \\(($qmgr_patt) \\) for reason ($reason_patt)\\. .* The problem is likely to be caused by the parent broker not yet existing, or a problem with the ($qname_patt) queue at the parent broker\\.",
                   "Broker", "Reason", "QName" ],
 
+   'AMQ5826' => [ "MQSeries message broker failed to propagate subscription to stream \\(($qname_patt) \\) at broker \\(($qmgr_patt) \\)\\. Reason codes ($reason_patt) and ($reason_patt)\\.",
+                  "Stream", "Broker", "Reason1", "Reason2" ],
+
    'AMQ5832' => [ "MQSeries message broker failed to publish configuration information on ($qname_patt)\\.",
                   "QName" ],
 
-   # FIXME: So far, encountered this guy without an actual broker name...
-   'AMQ5839' => [ "MQSeries message broker received unexpected inter-broker communication from broker \(\)" ],
+   'AMQ5833' => [ "A loop has been detected in the MQSeries message broker hierarchy\\." ],
 
-   'AMQ5840' => [ "MQSeries message broker unable to delete queue \\(($qname_patt)\\s*\\) for reason ($reason_patt)\\.",
+   'AMQ5834' => [ "Conflicting queue manager names in the MQSeries message broker hierarchy\\..* The names of the queue managers \\(($qmgr_patt)\\s*\\) and \\(($qmgr_patt)\\s*\\) in the MQSeries message broker hierarchy both start with the same 12 characters\\." ],
+
+   'AMQ5838' => [ "MQSeries message broker cannot be deleted as child \\(($qmgr_patt)\\s*\\) is still registered\\.",
+                  "Child" ],
+
+   # FIXME: So far, encountered this guy without an actual broker name...
+   'AMQ5839' => [ "MQSeries message broker received (?:an )?unexpected inter-broker communication" ],
+
+   'AMQ5840' => [ "MQSeries message broker (?unable to delete|failed to delete the) queue \\(($qname_patt)\\s*\\) for reason ($reason_patt)\\.",
                   "QName", "Reason" ],
 
    'AMQ5841' => [ "MQSeries message broker \\(($qmgr_patt)\\) deleted.",
@@ -108,8 +119,11 @@ $error_table =
    'AMQ5842' => [ "MQSeries message broker \\(($qmgr_patt)\\) cannot be deleted for reason '($reason_patt)'\\.",
                   "Broker", "Reason" ],
 
-   # FIXME: So far, enountered this guy without an actual relation name...
-   'AMQ5844' => [ "MQSeries message broker relation \\(\\) is unknown to broker \\(($qmgr_patt)\\)\\.",
+   'AMQ5843' => [ "MQSeries message broker \\(($qmgr_patt)\\) cannot be started as it is partially deleted\\.",
+                  "Broker" ],
+
+   # NOTE: This guy often has blank relation names, cannot be parsed reliably
+   'AMQ5844' => [ "MQSeries message broker relation \\(.*\\) is unknown to broker \\(($qmgr_patt)\\)\\.",
                   "Broker" ],
 
    'AMQ5847' => [ "MQSeries message broker \\(($qmgr_patt)\\) has removed knowledge of relation \\(($qmgr_patt)\\)\\.",
@@ -122,7 +136,12 @@ $error_table =
    'AMQ5849' => [ "MQSeries message broker \\(($qmgr_patt) \\) may not change parent from \\(($qmgr_patt) \\) to \\(($qmgr_patt) \\)\\.",
                   "Broker", "OldParent", "NewParent" ],
 
-   'AMQ5855' => [ "MQSeries message broker \\(($qmgr_patt)\\) ended for reason '($reason_patt)'\\.",
+   # NOTE: The 'related broker' can be a queue manager name or 
+   #       'unknown relation', so be generous in accepting that pattern
+   'AMQ5852' => [ "MQSeries message broker failed to propagate delete publication command for stream \\(($qname_patt)\\s*\\) to related broker \\(.*?\\s*\\) for reason ($reason_patt)\\.",
+                  "Stream", "Related", "Reason" ],
+
+   'AMQ5855' => [ "MQSeries message broker \\(($qmgr_patt)\\) (?:but the broker has )?ended for reason '($reason_patt)'\\.",
                   "Broker", "Reason" ],
 
    'AMQ5856' => [ "Broker publish command message cannot be processed\\. Reason code ($reason_patt)\\..* The MQSeries broker failed to process a publish message for stream \\(($qname_patt)\\s*\\)\\.",
@@ -140,6 +159,12 @@ $error_table =
    'AMQ5867' => [ " MQSeries message broker stream \\(($qname_patt)\\s*\\) has ended abnormally for reason ($reason_patt)\\.",
                   "Stream", "Reason" ],
 
+   'AMQ5869' => [ "MQSeries message broker is checkpointing registrations\\..*registrations of stream \\(($qname_patt)\\s*\\)\\.",
+                  "Stream" ],
+
+   # NOTE: The queue-manager name is missing here
+   'AMQ5876' => [ "A parent conflict has been detected in the MQSeries message broker hierarchy\\." ],
+
    'AMQ5878' => [ "MQSeries message broker recovery failure detected\\." ],
      
    # NOTE: The 'Stream' field here may look like 'uncreated stream',
@@ -151,6 +176,13 @@ $error_table =
    # Messages AMQ6000-6999 are Common Services Messages
    # Many of these (all?) should have a corresponding entry in the FDC logs.
    #
+   'AMQ6004' => [ "An error occurred during MQSeries initialization or ending\\." ],
+
+   'AMQ6026' => [ "A resource shortage prevented the creation of an MQSeries process\\." ],
+
+   'AMQ6035' => [ "MQSeries failed, no storage available\\." ],
+
+   'AMQ6037' => [ "MQSeries was unable to obtain enough storage\\." ],
 
    'AMQ6047' => [ "MQSeries is unable to convert string data tagged in CCSID \\d+ to data in CCSID \\d+\\." ],
 
@@ -179,7 +211,12 @@ $error_table =
 
    'AMQ6162' => [ "An error has occurred reading an INI file\\." ],
 
-   'AMQ6175' => [ "The system could not dynamically load the library ($exit_patt)",
+   'AMQ6166' => [ "The MQS.INI file or a queue manager QM.INI file is missing\\." ],
+
+   'AMQ6174' => [ "The library ($exit_patt) was not found\\.",
+                  "Exit" ],
+
+   'AMQ6175' => [ "The system could not dynamically load the library ($exit_patt)\. The error message",
                   "Exit" ],
 
    'AMQ6183' => [ "The failing process is process ($pid_patt)\\.",
@@ -191,19 +228,25 @@ $error_table =
    'AMQ6184' => [ "An internal MQSeries error has occurred on queue manager ($qmgr_patt)\\..*The failing process is process ($pid_patt)\\.",
                   "QMgr", "Pid" ],
 
+   'AMQ6209' => [ "An unexpected asynchronous signal \\((\\d+)\\) has been received and ignored\\..* Process ($pid_patt) received an unexpected asynchronous signal and ignored it\\.",
+                  "Signal", "Pid" ],
+
    # NOTE: So far, encountered this error without a location.
    'AMQ6708' => [ "A disk full condition was encountered when formatting a new log file in location (.*?)\\.",
                   "Location" ],
 
+   'AMQ6709' => [ "The log for the Queue manager is full\\." ],
+
    # 
    # Messages AMQ7000-7999 are MQSeries Product Messages
    #
-   'AMQ7030' => [ "Request to quiesce the queue manager accepted\\." ],
+   # AMQ7030 changed between MQ 5.0/5.1 and 5.2
+   'AMQ7030' => [ "(?:Request to quiesce the queue manager accepted\\.|Quiesce request accepted\\.)" ],
 
    'AMQ7075' => [ "Unknown attribute (\\S+) on line (\\d+) of ini file (\\S+)\\.",
                   "Attribute", "Line", "File" ],
 
-   'AMQ7076' => [ "Line (\\d+) of the configuration file (\\S+) contained value (.*?)\s*that is not valid for the attribute (\\S+)\\.",
+   'AMQ7076' => [ "Line (\\d+) of the configuration file (\\S+) contained value (.*?)\\s*that is not valid for the attribute (\\S+)\\.",
                   "Line", "File", "Value", "Attribute" ],
 
    'AMQ7159' => [ "A FASTPATH application has ended unexpectedly" ],
@@ -213,6 +256,15 @@ $error_table =
 
    'AMQ7463' => [ "The log for queue manager ($qmgr_patt) is full\\.",
                   "QMgr" ],
+   
+   'AMQ7466' => [ "The log for queue manager ($qmgr_patt) is too small to support the current data rate\\.",
+                  "QMgr" ],
+
+   'AMQ7467' => [ "The oldest log file required to start queue manager ($qmgr_patt) is ($logfile_patt)\\.",
+                  "QMgr", "LogFile" ],
+
+   'AMQ7468' => [ "The oldest log file required to perform media recovery of queue manager ($qmgr_patt) is ($logfile_patt)\\.",
+                  "QMgr", "LogFile" ],
 
    'AMQ7469' => [ "Transactions rolled back to release log space\\." ],
 
@@ -236,8 +288,8 @@ $error_table =
 
    'AMQ7924' => [ "Bad length in the PCF header \\(length = \\d+\\)\\." ],
 
-   # FIXME: If encountered again, determine message version pattern
-   'AMQ7925' => [ "Message version 16777216 is not supported" ],
+   'AMQ7925' => [ "Message version (\\d+) is not supported",
+                  "MessageVersion" ],
 
    # FIXME: If encountered again, determine CCSID pattern
    'AMQ7935' => [ "Bad CCSID in message header \\(CCSID = 0\\)" ],
@@ -255,6 +307,8 @@ $error_table =
    # FIXME: Are the number of objects of interest?
    'AMQ8048' => [ "Default objects statistics :" ],
 
+   'AMQ8214' => [ "CONNAME parameter required with channel types SDR, RQSTR, CLNTCONN, CLUSSDR and CLUSRCVR\\." ],
+
    'AMQ8226' => [ "MQSeries channel ($channel_patt) cannot be created",
                   "Channel" ],
 
@@ -307,23 +361,39 @@ $error_table =
    'AMQ9209' => [ "An error occurred receiving data from '($host_patt)' over TCP/IP\\.", 
                   'Host' ],
 
-   'AMQ9213' => [ "The return code from the TCP/IP \\((.*?)\\) call was ($rc_patt) \\(X'", 
+   'AMQ9213' => [ "The return code from the (.*?) call was ($rc_patt) \\(X'", 
                   "Operation", "IPCode" ],
 
+   'AMQ9220' => [ "The attempt to load the TCP/IP library or procedure '(.*?)' failed with error code ($rc_patt)\\.",
+                  "Library", "Reason" ],
+
    'AMQ9221' => [ " The specified value of '(.*?)' was not recognized as one of the protocols supported\\.",
                   "Protocol" ],
    
    'AMQ9228' => [ "The TCP/IP responder program could not be started\\." ],
 
+   'AMQ9243' => [ "The queue manager '($qmgr_patt)' does not exist\\.",
+                  "QMgr" ],
+
    'AMQ9245' => [ "MQSeries was unable to obtain the account details for MCA user ID '(\\w+)'\\. This user ID was the MCA user ID for channel '($channel_patt)' on queue manager '($qmgr_patt)' and may have been defined in the channel definition, or supplied either by a channel exit or by a client\\.",
                   "Userid", "Channel", "QMgr" ],
 
+   'AMQ9409' => [ "The repository manager ended abnormally\\." ],
+
    'AMQ9410' => [ "The repository manager started successfully\\." ],
 
    'AMQ9411' => [ "The repository manager (?:ended|stopped) normally\\." ],
 
+   'AMQ9422' => [ "Repository manager error, RC=(\\d+)",
+                  "Reason" ],
+
+   'AMQ9437' => [ "Unable to commit repository changes.\\" ],
+
    'AMQ9447' => [ "Following an error, the repository manager tried to backout some updates to the repository, but was unsuccessful\\. The repository manager terminates\\." ],
 
+   'AMQ9502' => [ "Type of channel not suitable for action requested\\..*The operation requested cannot be performed on channel '($channel_patt)'\\.",
+                  "Channel" ],
+
 
    # NOTE: At time, this gets invalid channel names (eg, from bad clients),
    #       so we use a non-standard pattern
@@ -371,6 +441,9 @@ $error_table =
                    "Channel" ],
 
    'AMQ9522' => [ "The program could not access the channel status table\\." ],
+
+   'AMQ9523' => [ "During communications through channel '($channel_patt)', the remote queue manager channel program detected a protocol error\\.",
+                  "Channel" ],
    
    'AMQ9524' => [ "Channel '($channel_patt)' cannot start because the remote queue manager is not currently available\\.",
                   "Channel" ],
@@ -405,6 +478,9 @@ $error_table =
    'AMQ9536' => [ "Channel program '($channel_patt)' was ended by exit '($exit_patt)'",
                   "Channel", "Exit" ],
 
+   'AMQ9540' => [ "The program ended because return code ($code_patt) was received when an attempt was made to commit change to the resource managers. The commit ID was '(.*?)'\\.",
+                  "Reason", "CommitId" ],
+
    'AMQ9542' => [ "Queue manager is ending\\." ],
 
    'AMQ9544' => [ "During the processing of channel '($channel_patt)' one or more.*The program identifier \\(PID\\) of the processing program was '($pid_patt)'\\.",
